﻿#region Header

// Copyright (c) Dundas Data Visualization Inc. All rights reserved.
//
// THIS SOURCE CODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT ANY
// WARRANTY. IT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE
// QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE
// DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
//
// IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW THE AUTHOR WILL BE LIABLE TO YOU
// FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT
// LIMITED TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
// YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
// PROGRAMS), EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.

#endregion Header

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Web;
using System.Web.Http;

using Dundas.BI;
using Dundas.BI.AccountServices;
using Dundas.BI.WebApi.Controllers;
using Dundas.BI.WebApi.Models;

using Newtonsoft.Json;

namespace CustomApiController
{
    /// <summary>
    /// 
    /// </summary>
    /// <seealso cref="System.Web.Http.ApiController" />
    public class HelloWorldController : ApiController


	{

#region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="HelloWorldController"/> class.
        /// </summary>
        public HelloWorldController() { }

#endregion Constructors

#region Private Methods

	    private Guid? GetSessionIdFromCookieOrProvided(Guid? sessionIdFromQueryString)
	    {
		    // Check if it's provided.
		    if (sessionIdFromQueryString.HasValue
			    && sessionIdFromQueryString.Value != Guid.Empty)
		    {
			    return sessionIdFromQueryString;
		    }

		    // Try to get it from the cookie.
		    string cookieName = "dundas_webapp_sessionid";
		    CookieHeaderValue cookie = this.Request.Headers.GetCookies(cookieName).FirstOrDefault();
		    if (cookie != null)
		    {
			    Guid result;
			    if (Guid.TryParse(cookie[cookieName].Value, out result)
				    && result != Guid.Empty)
			    {
				    return result;
			    }
		    }

		    return null;
	    }

	    private CallerContext Session(Guid? sessionId)
	    {
		    ICallerContextService callerContextService = Engine.Current.GetService<ICallerContextService>();

		    // If an empty GUID is passed, there is no session, so we need to pass null when creating the context.
		    Guid? sessIdForContext = this.GetSessionIdFromCookieOrProvided(sessionId);

		    // Store this session ID in case we need it for exception handling (it may not be the current session).
		    this.StoreSessionIdInHttpContext(sessIdForContext);

		    // Return the context.
		    return callerContextService.CreateAndSetCurrentContext(sessIdForContext);
	    }

	    private void StoreSessionIdInHttpContext(Guid? sessionId)
	    {

			// Used to associate exceptions with a session.
			if (System.Web.HttpContext.Current != null)
			{
				System.Web.HttpContext.Current.Items["sessionId"] = sessionId;
			}
		}


#endregion Private Methods

#region Public Methods

		/// <summary>
		/// <c>POST /API/HelloWorld/</c>
		/// Says hello.
		/// </summary>
		/// <param name="options">A JSON string containing the options for the transfer.</param>
		/// <param name="sessionId">The current session ID.</param>
		/// <returns>A <see cref="System.String"/> containing the hello world string.</returns>
		[HttpPost]
	    [ActionName("Index")]
	    public HttpResponseMessage Index([FromBody]dynamic options, Guid? sessionId = null)
	    {
		    // Breakpoint for testing for correct mapping.
		    // System.Diagnostics.Debugger.Launch();

		    if (!this.ModelState.IsValid || options == null)
		    {
			    return this.Request.CreateErrorResponse(HttpStatusCode.BadRequest, this.ModelState);
		    }

		    using (this.Session(sessionId))
		    {
			    try
			    {

				    Session session = Engine.Current.Session;

				    var response = this.Request.CreateResponse(HttpStatusCode.OK);

				    if (options.WithUsername == true)
				    {
					    response.Content =
						    new StringContent(
							    string.Format(
								    CultureInfo.CurrentCulture,
								    "Hello {0}!",
								    session.AccountDisplayName
								    )
							    );
				    }
				    else
				    {

					    response.Content =
						    new StringContent(
								    "Hello!"
							    );

				    }
				    return response;
			    }
			    catch (ArgumentException ex)
			    {
				    return this.Request.CreateResponse(HttpStatusCode.BadRequest, ex);
			    }
			    catch (InvalidOperationException ex)
			    {
				    return this.Request.CreateResponse(HttpStatusCode.MethodNotAllowed, ex);
			    }
			    catch (NoPrivilegeException ex)
			    {
				    return this.Request.CreateResponse(HttpStatusCode.Forbidden, ex);
			    }
			    catch (InvalidSessionException ex)
			    {
				    return this.Request.CreateErrorResponse((HttpStatusCode)Dundas.BI.WebApi.WebApiConstants.InvalidSessionHttpStatusCode, ex);
			    }
		    }
	    }

#endregion Public Methods
    }
}